/*****************************************************************************
 HMS Technology Center Ravensburg GmbH
******************************************************************************

 Project : CANCON.EXE
 File    : CANCON.C
 Summary : Demo application for the IXXAT VCI C-API.
             
 Date    : 2005-07-28

 Compiler: Microsoft Visual C++

 Remarks : This demo demonstrates the following VCI features
           - adapter selection
           - controller initialization 
           - creation of a message channel
           - transmission / reception of CAN messages
******************************************************************************
 all rights reserved
*****************************************************************************/

/*****************************************************************************
 * include files
 ****************************************************************************/

#include <vcinpl.h>

#include <process.h>
#include <stdio.h>
#include <conio.h>
#include <tchar.h>

#include "select.hpp"


/*****************************************************************************
 * global variables
 ****************************************************************************/

static HANDLE          hDevice;     // handle to selected VCI device
static LONG            lCtrlNo;     // number of selected CAN controller
static HANDLE          hCanCtl;     //  handle to selected CAN controller
static CANCAPABILITIES sCanCaps;    // capabilities of selected CAN controller

static HANDLE          hCanChn;     // handle to CAN channel
static volatile LONG   lEndRxT = 0; // quit flag for the receive thread


/*************************************************************************
 * local functions
*************************************************************************/

HRESULT SelectDevice ( BOOL fUserSelect );
HRESULT InitSocket   ( UINT32 dwCanNo );
void    FinalizeApp  ( void );
void    DisplayError ( HRESULT hResult );

HRESULT TransmitData ( void );
void    ReceiveThread( void* Param );


/*****************************************************************************
 Function:
  main

 Description:
  Main entry point of the application.

 Arguments:
  none

 Results:
  none
*****************************************************************************/
void main(void)
{
  HRESULT hResult;

  _tprintf(TEXT(" >>>> VCI - C-API Example V3.0 <<<<\n"));
  _tprintf(TEXT("\n initializes the CAN with 125 kBaud"));
  _tprintf(TEXT("\n sends CAN messages with ID 100h"));
  _tprintf(TEXT("\n shows all received messages"));
  _tprintf(TEXT("\n\n"));

  _tprintf(TEXT(" Select Adapter..."));
  hResult = SelectDevice( TRUE );
  _tprintf(TEXT("%s\n\n"),
    SUCCEEDED(hResult)? TEXT("done!"): TEXT("failed!"));

  if (hResult == VCI_OK)
  {
    _tprintf(TEXT(" Initialize CAN..."));
    hResult = InitSocket( lCtrlNo );
    _tprintf(TEXT("%s\n\n"),
      SUCCEEDED(hResult)? TEXT("done!"): TEXT("failed!"));
  }

  if (hResult == VCI_OK)
  {
    BOOL fQuit = FALSE;

    _tprintf(TEXT("\n--------------------------------------------------"));
    _tprintf(TEXT("\n press [t] to send a message with ID 100h"));
    _tprintf(TEXT("\n press [Esc] to quit the application"));
    _tprintf(TEXT("\n--------------------------------------------------\n"));

    //
    // start the receive thread
    //
    _beginthread(ReceiveThread, 0, NULL);

    //
    // wait for a keyboard hit to transmit CAN-messages
    //

    while(!fQuit)
    {
      //
      // wait for the user to press a key
      //
      switch (_getch())
      {
        // send a message if the user presses the [t] key
        case 't':
        case 'T':
          TransmitData();
          break;

        // terminate the program if the user presses the [ESC] key
        case VK_ESCAPE:
          fQuit = TRUE;
          break;
      }
    }

    //
    // tell receive thread to quit
    //
    InterlockedExchange(&lEndRxT, 1);
  }
  else
  {
    DisplayError(hResult);
  }

  _tprintf(TEXT("\n--------------------------------------------------"));
  _tprintf(TEXT("\n Free resources..."));
  FinalizeApp();
  _tprintf(TEXT("done!\n\n"));
}

/*****************************************************************************
 Function:
  SelectDevice

 Description:
  Selects the first CAN adapter.

 Arguments:
  fUserSelect -> If this parameter is set to TRUE the functions display
                 a dialog box which allows the user to select the device.

 Results:
  If the function succeeds it returns VCI_OK. If the function fails it
  returns an error code other than VCI_OK.
*****************************************************************************/
HRESULT SelectDevice( BOOL fUserSelect )
{
  HRESULT hResult; // error code

  if (fUserSelect == FALSE)
  {
    HANDLE        hEnum; // enumerator handle
    VCIDEVICEINFO sInfo; // device info

    //
    // open the device list
    //
    hResult = vciEnumDeviceOpen(&hEnum);

    //
    // retrieve information about the first
    // device within the device list
    //
    if (hResult == VCI_OK)
    {
      hResult = vciEnumDeviceNext(hEnum, &sInfo);
    }

    //
    // close the device list (no longer needed)
    //
    vciEnumDeviceClose(hEnum);

    //
    // open the device
    //
    if (hResult == VCI_OK)
    {
    #if defined(__cplusplus)
      hResult = vciDeviceOpen(sInfo.VciObjectId, &hDevice);
    #else
      hResult = vciDeviceOpen(&sInfo.VciObjectId, &hDevice);
    #endif
    }

    //
    // select controller 0
    //
    lCtrlNo = 0;
  }
  else
  {
    //
    // open a device and controller selected by the user
    //
    hResult = SocketSelectDlg(NULL, VCI_BUS_CAN, &hDevice, &lCtrlNo);
  }

  return( hResult );
}

/*************************************************************************
 Function:
  InitSocket

 Description:
  Creates a CAN message channel, initializes and starts the CAN
  controller.

 Arguments:
  dwCanNo -> Number of the CAN controller to open.

 Results:
  If the function succeeds it returns VCI_OK. If the function fails
  it returns an error code other than VCI_OK.

 Remarks:
  If <dwCanNo> is set to 0xFFFFFFFF, the function shows a dialog box
  which allows the user to select the VCI device and CAN controller.
*************************************************************************/
HRESULT InitSocket(UINT32 dwCanNo)
{
  HRESULT hResult;

  if (hDevice != NULL)
  {
    //
    // create and initialize a CAN message channel
    //
    hResult = canChannelOpen(hDevice, dwCanNo, FALSE, &hCanChn);

    if (hResult == VCI_OK)
    {
      hResult = canChannelInitialize(hCanChn,
                  1024, // size of receive FIFO in messages
                  1,    // threshold for receive FIFO event
                  128,  // size of transmit FIFO in messages
                  1);   // threshold for transmit FIFO event

    }

    //
    // activate the CAN channel
    //
    if (hResult == VCI_OK)
    {
      hResult = canChannelActivate(hCanChn, TRUE);
    }

    //
    // open and initialize the CAN controller
    //
    if (hResult == VCI_OK)
    {
      hResult = canControlOpen(hDevice, dwCanNo, &hCanCtl);
      // this function fails if the controller is in use
      // by another application

      //
      //  request the capabilities of the selected controller
      //
      if (hResult == VCI_OK)
      {
        hResult = canControlGetCaps(hCanCtl, &sCanCaps);
      }

      //
      // initialize the selected CAN controller
      //
      if (hResult == VCI_OK)
      {
        UINT8 bOpMode = CAN_OPMODE_UNDEFINED;

        if (sCanCaps.dwFeatures & CAN_FEATURE_STDANDEXT)
        {
          // controller supportes CAN_OPMODE_STANDARD and
          // CAN_OPMODE_EXTENDED at the same time
          bOpMode |= CAN_OPMODE_STANDARD; // enable reception of 11-bit id messages
          bOpMode |= CAN_OPMODE_EXTENDED; // enable reception of 29-bit id messages
        }
        else if (sCanCaps.dwFeatures & CAN_FEATURE_STDOREXT)
        {
          // controller supportes either CAN_OPMODE_STANDARD or
          // CAN_OPMODE_EXTENDED but not both at the same time
          bOpMode |= CAN_OPMODE_STANDARD; // enable reception of 11-bit ID messages
        //bOpMode |= CAN_OPMODE_EXTENDED; // enable reception of 29-bit ID messages
        }

        if (sCanCaps.dwFeatures & CAN_FEATURE_ERRFRAME)
        {
          // controller supports reception of error frames
          bOpMode |= CAN_OPMODE_ERRFRAME; // enable reception of error frames
        }

        hResult = canControlInitialize(hCanCtl,
                    bOpMode,         // CAN operating mode
                    CAN_BT01_125KB); // bit rate

        //
        // set the controller specific acceptance filter for 11-bit IDs
        //
        if ((hResult == VCI_OK) && (bOpMode & CAN_OPMODE_STANDARD))
        {
          hResult = canControlSetAccFilter(hCanCtl,
             CAN_FILTER_STD, CAN_ACC_CODE_ALL, CAN_ACC_MASK_ALL);
        }

        //
        // set the controller specific acceptance filter for 29-bit IDs
        //
        if ((hResult == VCI_OK) && (bOpMode & CAN_OPMODE_EXTENDED))
        {
          hResult = canControlSetAccFilter(hCanCtl,
             CAN_FILTER_EXT, CAN_ACC_CODE_ALL, CAN_ACC_MASK_ALL);
        }
      }
    }

    //
    // start the CAN controller
    //
    if (hResult == VCI_OK)
    {
      hResult = canControlStart(hCanCtl, TRUE);
    }
  }
  else
  {
    hResult = VCI_E_INVHANDLE;
  }

  return( hResult );
}

/*************************************************************************
 Function: 
  TransmitData

 Description:
  Transmits a CAN message with ID 0x100.

 Arguments:
  none

 Results:
  none
*************************************************************************/
HRESULT TransmitData(void)
{
  HRESULT hResult;
  UINT8   bDataLen = 8; // select data length
  CANMSG  sCanMsg;
  UINT8   i;

  if (sCanCaps.dwFeatures & CAN_FEATURE_DELAYEDTX)
  { // controller supports delayed transmission
    sCanMsg.dwTime = 10;  // delay by 10 milliseconds
  }
  else
  {
    sCanMsg.dwTime = 0;
  }

  sCanMsg.dwMsgId = 0x100; // CAN message identifier

  sCanMsg.uMsgInfo.Bytes.bType  = CAN_MSGTYPE_DATA;
  sCanMsg.uMsgInfo.Bytes.bFlags = 0;
  sCanMsg.uMsgInfo.Bytes.bFlags2 = 0;

  sCanMsg.uMsgInfo.Bytes.bAccept = 0;
  sCanMsg.uMsgInfo.Bytes.bReserved = 0;

//sCanMsg.uMsgInfo.Bits.srr = 1;  // self reception request
//sCanMsg.uMsgInfo.Bits.rtr = 1;  // remote transmission request
//sCanMsg.uMsgInfo.Bits.ext = 1;  // select 29 bit ID

  sCanMsg.uMsgInfo.Bits.dlc = bDataLen;  // data length

  //
  // initialize data fields
  //
  for (i = 0; i < bDataLen; i++ )
  {
    sCanMsg.abData[i] = i;
  }

  //
  // clear unused data fields
  //
  for (i = bDataLen; i < NUMELEM(sCanMsg.abData); i++ )
  {
    sCanMsg.abData[i] = 0;
  }

  //
  // write the CAN message into the transmit FIFO
  //
  hResult = canChannelSendMessage(hCanChn,
              100, // wait for at least 100 milliseconds before the
                   // function returns even if no message has been
                   // written to the transmit FIFO
              &sCanMsg);

  return( hResult );
}

#pragma warning(disable:4100) // unreferenced formal parameter

/*************************************************************************
 Function: 
  ReceiveThread

 Description: 
  Receive thread.

 Arguments: 
  none

 Results:
  none
*************************************************************************/
void ReceiveThread( void* Param )
{
  HRESULT hResult;
  CANMSG  sCanMsg;

  while ( lEndRxT == 0 )
  {
    // read a CAN message from the receive FIFO

    hResult = canChannelReadMessage(hCanChn, 100, &sCanMsg);

    if (hResult == VCI_OK)
    {
      if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_DATA)
      {
        //
        // show data frames
        //
        if (sCanMsg.uMsgInfo.Bits.rtr == 0)
        {
          UINT8 j;

          _tprintf(TEXT("\nTime: %10u  ID: %3X  DLC: %1u  Data:"),
            sCanMsg.dwTime, sCanMsg.dwMsgId, sCanMsg.uMsgInfo.Bits.dlc);

          for (j = 0; j < sCanMsg.uMsgInfo.Bits.dlc; j++)
          {
            _tprintf(TEXT(" %.2X"), sCanMsg.abData[j]);
          }
        }
        else
        {
          _tprintf(TEXT("\nTime: %10u ID: %3X  DLC: %1u  Remote Frame"),
            sCanMsg.dwTime, sCanMsg.dwMsgId, sCanMsg.uMsgInfo.Bits.dlc);
        }
      }
      else if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_INFO)
      {
        //
        // show informational frames
        //
        switch (sCanMsg.abData[0])
        {
          case CAN_INFO_START:
            _tprintf(TEXT("\nCAN started..."));
            break;

          case CAN_INFO_STOP:
            _tprintf(TEXT("\nCAN stoped..."));
            break;

          case CAN_INFO_RESET:
            _tprintf(TEXT("\nCAN reseted..."));
            break;
        }
      }
      else if (sCanMsg.uMsgInfo.Bytes.bType == CAN_MSGTYPE_ERROR)
      {
        //
        // show error frames
        //
        switch (sCanMsg.abData[0])
        {
          case CAN_ERROR_STUFF:
            _tprintf(TEXT("\nstuff error..."));
            break;

          case CAN_ERROR_FORM:
            _tprintf(TEXT("\nform error..."));
            break;

          case CAN_ERROR_ACK:
            _tprintf(TEXT("\nacknowledgment error..."));
            break;

          case CAN_ERROR_BIT:
            _tprintf(TEXT("\nbit error..."));
            break;

          case CAN_ERROR_CRC:
            _tprintf(TEXT("\nCRC error..."));
            break;

          case CAN_ERROR_OTHER:
          default:
            _tprintf(TEXT("\nother error..."));
            break;
        }
      }
    }
  }

  _endthread();
}

#pragma warning(default:4100) // unreferenced formal parameter

/*************************************************************************
 Function: 
  FinalizeApp

 Description:
  Finalizes the application

 Arguments:
  none
  
 Results:
  none
*************************************************************************/
void FinalizeApp()
{
  //
  // close all open handles
  //
  canChannelClose(hCanChn);
  canControlClose(hCanCtl);
  vciDeviceClose(hDevice);
}

/*************************************************************************
 Function: 
  DisplayError

 Description:
  This function displays a message box for the specified error code.

 Arguments:
  hResult -> Error code or -1 to display the error code returned
             by GetLastError().

 Results:
  none
*************************************************************************/
void DisplayError(HRESULT hResult)
{
  TCHAR szError[1024];

  if (hResult != NO_ERROR)
  {
    if (hResult == -1)
      hResult = GetLastError();

    szError[0] = 0;
    vciFormatError(hResult, szError, NUMELEM(szError));
    MessageBox(NULL, szError, TEXT("VCI Demo"), MB_OK | MB_ICONSTOP);
  }
}
